###################################################################
# file name: restore_my_flash.pl       (restore flash on dev kits)
# Marcus Negron  Altera, SCTC                    date: 4\21\2004
# Kenneth Wong   Microtronix Datacom, Ltd.       date: 10\08\2004
# JRK            Altera, SCTC                    date: 11-Oct-2005
#
#    USAGE: ./restore_my_flash [<kit_name>]
#
# 10-Mar-2005: SPR 166904 -- Separated shell script created from
#              original perl to allow proper operation on both
#              PCs & Linux.
#
# 11-Oct-2005: SPR 187057 -- Updated for operation with new
#              nios2-flash-programmer utility & unified support
#              for all dev boards in one script.
###################################################################

use strict;
use ptf_parse;

# set up some global variables

my $base               = "0x0";     # CFI flash base address (standard design default)
my $epcs_base          = "0x800800";# EPCS flash base address (1c12 stadard design default)

my $restore_flash      = "";         # place saver for "the" cat flash file
my $restore_epcs_flash = "";         # place saver for "the" epcs flash file
my $target_sof         = "";         # place saver for "the" target sof file

my $board_reconfig_sw  = "SW9 button in the \"FPGA Config\" Box";# default push button to load factory design
my $n2fp               = "nios2-flash-programmer";   # short comand string
my $qpgm               = "quartus_pgm";              # command string for Quartus Programmer
my $short_kit_name     = "";                         # kit name for programming
my $full_kit_name      = "";                         # kit name for printing
my $user_doc           = "Appendix B of the Nios II Development Board Reference Manual"; # guide location
my $cable_instructions = "that your programming cable is connected properly
 (polarity?) to J24";                # how to connect your cable on most boards


# sub routines

sub display_usage
{
printf ("\n  restore_my_flash.pl

  This program accepts the specific name of an Altera Nios II Development Kit,
  as listed below and restores the default runtime code and \"Factory
  Configuration\" by \"flash_programming\" it back into your development board.

  If the flash files that were shipped with the kit have been written over or
  otherwise missing, flash contents will be restored to the \"out-of-box\"
  state.

  If you do not enter a \"kit_name\" this program will search in the immediate
  directory to try to determine which kit to restore by default.

  In either case you will be prompted to confirm kit restoration or try running
  the program correctly.

  If you do not enter a \"kit_name\" and the program cannot determine a default
  \"kit_name\" you should try again specifying a \"kit_name\".

  Please NOTE that this script currently must be run with the prepending \"\./\"
  directive for the script to run correctly.

    USAGE: ./restore_my_flash [<kit_name>]

     kit_name : Nios II Development Kit Name

       Options:

     cyclone_1c20        =  Nios II Development Kit, Cyclone 1C20 Edition
     stratix_1s10        =  Nios II Development Kit, Stratix 1S10 Edition
     stratix_1s10_ES     =  Nios II Development Kit, Stratix 1S10ES Edition
     stratix_1s40        =  Nios II Development Kit, Stratix 1S40 Edition
     stratixII_2s60_ES   =  Nios II Development Kit, Stratix II 2S60ES Edition
     stratixII_2s60_rohs =  Nios II Development Kit, Stratix II 2S60 (rohs) Ed.
     stratixII_2s60      =  Nios II Development Kit, Stratix II 2S60 Edition
     cycloneII_2c35      =  Nios II Development Kit, Cyclone II 2C35 Edition
     cyclone_1c12_eval   =  Nios II Evaluation Kit, Cyclone 1C12 Edition
     
   Example1 (explicit argument) :
   
   [SOPC Builder]\$ \./restore_my_flash stratix_1s10_ES <<-Enter>>
   
   Will systematically restore all contents shipped
   on a Nios II Development Kit, Stratix 1S10ES Edition.

   Example2 (default argument) :

   [SOPC Builder]\$/cygdrive/c/altera/kits/nios2/examples/factory_recovery
                /niosII_cyclone_1c20/ ./restore_my_flash  <<-Enter>>

   Will systematically restore all contents shipped
   on a Nios II Development Kit, Cyclone 1C20 Edition.

     ");

   return;
}


sub check_arg        # return $kit_name_entered= ok, 0 = incorrect usage (default search)
{
  my $kit_name_entered= "";    # user selection goes here
  my $arg_count     = @ARGV;   # assign number of args to argCount

  # check for command line errors
  if ($arg_count > 1) # check for wrong argument
  {
    display_usage;              # display usage message to user
    return 0;
  }
  else
  {
    $kit_name_entered     = $ARGV[0];  # 1st arg is dev kit name
    return $kit_name_entered;
  }

}


sub find_kit_name                       # success = 0, failure = kit name
{
  my $locDir           = $ENV{"PWD"};   # environment\print working directory
  my $kit_name_entered = check_arg;     # grab user command arguments
  my $kit_name_determined = "";		# put kit name determined by script here
  
    # first figure out the kit name by matching against the working directory?
    if ($locDir =~ /(niosII_)(\w+)/)
    {
      $kit_name_determined = $2; # get the chip name for the kit name

      if ( ($kit_name_determined eq $kit_name_entered) or ( $kit_name_entered eq "" ) )
      {
        return $kit_name_determined; # got something
      }
      else
      {
        print "\n (ERROR): Determined kit name \"$kit_name_determined\" (from current
          working directory) is NOT the same as kit entered:
         \"$kit_name_entered\"\n";
      }
    }
    elsif ($kit_name_entered)
    {
      return $kit_name_entered;
    }

  print "\n (ERROR): Could not determine kit name based on current directory or arguments.\n";

  return 0; # early withdrawl
}
 
 
sub display_kit_name_error    
{
    print "\n RESTORE ABORTED: 
    Could not find correct Nios II Development Kit Name!
    Nios II Development Kit Name entered was \"$short_kit_name\"
    Nios II Development Kit Name might be incorrect?\n
    Suggestion: Please check the \"kit_name\" entered at the
    command line and try again.\n\n";

 return;
}
 
 
sub set_kit_vars                 # get kit variables. success = 1, failure = 0
{
  print " Running flash restore on Nios II Development Kit: $short_kit_name\n\n";
	
  if ($short_kit_name =~ /^cyclone_1c20$/)
  {
    $full_kit_name = "Nios II Development Kit, Cyclone 1C20 Edition";
  }
  elsif ($short_kit_name =~ /^stratix_1s40$/)
  {
    $full_kit_name = "Nios II Development Kit, Stratix 1S40 Edition";
  }
  elsif ($short_kit_name =~ /^stratix_1s10_ES$/)
  {
    $full_kit_name = "Nios II Development Kit, Stratix 1S10ES Edition";
  }
  elsif ($short_kit_name =~ /^stratix_1s10$/)
  {
    $full_kit_name = "Nios II Development Kit, Stratix 1S10 Edition";
  }
  elsif ($short_kit_name =~ /^stratixII_2s60_ES$/)
  {
    $full_kit_name = "Nios II Development Kit, Stratix II 2S60ES Edition";
  }
  elsif ($short_kit_name =~ /^stratixII_2s60_rohs$/)
  {
    $full_kit_name = "Nios II Development Kit, Stratix II 2S60 (rohs) Edition";
  }
  elsif ($short_kit_name =~ /^stratixII_2s60$/)
  {
    $full_kit_name = "Nios II Development Kit, Stratix II 2S60 Edition";
  }
  elsif ($short_kit_name =~ /^cycloneII_2c35$/)
  {
    $full_kit_name = "Nios II Development Kit, Cyclone II 2C35 Edition";
  }
  elsif ($short_kit_name =~ /^cyclone_1c12_eval$/)
  {
    $full_kit_name      = "Nios II Evaluation Kit, Cyclone 1C12 Edition";
    $board_reconfig_sw  = "SW1 button";
    $cable_instructions = "that your USB cable is properly connected to P2";
  }
  else
  {
    return 0; # early withdrawl
  }

 return $full_kit_name;
}


# Prompt for input
# ----------------
# Two arguments, prompt, and default value
sub prompt
{
  my ($prompt, $default) = @_;
  my $x;

  $prompt .= ":" if $prompt =~ /.*[a-zA-Z]/;
  print "$prompt [$default] ";
  $x = <STDIN>;
  chop($x);
  $x = $default unless $x;
  $x = "" if $x eq " ";

  return $x;
}

sub print_likely_cause_of_programming_failure
{
  print "
 Please be sure you are running this script from the appropriate
 kit directory, $cable_instructions, and try again.\n";
}

#
# For some bizarre reason executing a command with the conventional
# "`<command>`" or "open(CMD, <command>|)" methods produces flaky
# results on linux. This routine re-directs STDOUT to a temporary
# file, executes using "system()", and cleans up after itself.
#
sub handle_execution
{
  my $command = shift;

  # a sensible place to stash stdout
  my $temp_file = $ENV{"HOME"}."/restore_my_flash.tmp";
  my $results   = "";

  # May as well attempt to educate the user
  print " STATUS: executing command:
         $command\n\n";

  # Save copy of STDOUT in preparation for below.
  open OLDOUT, ">&STDOUT" or die "Can't save STDOUT: $!";

  # Redirect STDOUT and STDERR to log file.
  open STDOUT, '>', $temp_file or die "Can't redirect STDOUT $!";

  # Make unbuffered
  select STDOUT; $| = 1;

  # Run command -- we don't care about return codes; will parse STDOUT
  system($command);

  # Restore STDOUT to original state.
  open STDOUT, ">&OLDOUT" or die "Can't restore STDOUT to OLDOUT: $!";

  # Print out & save results of command
  print "-----------------------------------------\n";
  if (open(REDIRECTED_LOG_DISPLAY, $temp_file))
  {
    while (<REDIRECTED_LOG_DISPLAY>) {
      $results .= $_;   # capture a line of output
      print;            # echo command ouput to user
    }

    # Clean up
    close(REDIRECTED_LOG_DISPLAY);
    unlink("$temp_file");
  }
  print "-----------------------------------------\n\n";

  return $results;
}

# Pay attention while rescuing, to detect and warn users about errors
sub do_command
{
  # string = command and arguments
  my ($command)        = (@_);

  # success messages
  my $n2fp_success     = "Device contents checksummed OK";
  my $n2fp_epcs_success= "Programmed";
  my $n2fp_no_change   = "No change to device contents";
  my $qpgm_success     = "Quartus II Programmer was successful";

  # failure messages:
  # base address for flash programming wasn't correct
  my $no_cfi_table_err  = "No CFI table found at address";
  my $no_epcs_regs_err  = "No EPCS registers found";
  my $bad_cfi_addr_err  = "Base address not aligned on size of device";

  # expected .sof or .flash file not found
  my $n2fp_file_err     = "$restore_flash: Unable to open input file";
  my $qpgm_file_err     = "Error: File name \"$target_sof\" does not exist";

  # No cable attached
  my $n2fp_no_cable_err = "There are no JTAG cables available";
  my $qpgm_no_cable_err = "Error: Programming hardware cable not detected";

  # Cable not connected to board properly
  my $qpgm_scan_err     = "Error: Can't scan JTAG chain. Error code 87";

  # multiple cable error
  my $n2fp_cable_err    = "There are two or more JTAG cables";
  my $qpgm_cable_err    = "Error: More than one programming cable found";

  # wrong FPGA for .sof file
  my $qpgm_id_err       = "Can't configure device. Expected JTAG ID code";

  # error flag; assume the worst!
  my $err_flag          = 1;

  #okay, go run the darn thing
  my $results = handle_execution($command);

  # read the file and search for success or errors

   # pass flags and no error flag?
   if ( ($results =~ /$n2fp_success/)      ||
        ($results =~ /$n2fp_no_change/)    ||
        ($results =~ /$n2fp_epcs_success/)    )
   {
     print " STATUS: Flash programming operation successful...\n";
     $err_flag = 0; # clear error flag
   }
   elsif ($results =~ /$qpgm_success/)
   {
     print " STATUS: SOF Download Successful...\n";
     $err_flag = 0; # clear error flag
   }
   # base address error
   elsif ( ($results =~ /$no_cfi_table_err/) ||
           ($results =~ /$no_epcs_regs_err/) ||
           ($results =~ /$bad_cfi_addr_err/)    )
   {
     print "\n ERROR: BAD BASE ADDRESS ERROR.\n";
     print "\n Please make sure you have the correct kit directory and\n";
     print "   that the factory-restore .sof file is properly downloaded\n";
   }
   # UH, OOOOH, definitely a missing sof/flash file.
   elsif ( ($results =~ /$n2fp_file_err/) || ($results =~ /$qpgm_file_err/))
   {
     print "\n ERROR: INPUT FILE ERROR : Missing sof/flash file?\n";
     print_likely_cause_of_programming_failure();
   }
   # less likely mistake, multiple cables
   elsif ( ($results =~ /$n2fp_cable_err/) || ($results =~ /$qpgm_cable_err/) )
   {
     print "
     ERROR: MULTIPLE PROGRAMMING CABLE ERROR:
     
     Please check to be sure that ONLY ONE programming cable 
       is attached to your JTAG cable system and try again. 
     
     SUGGESTION: Type \"jtagconfig\" to list the attached cables, 
       then use \"jtagconfig --remove <cable number>\" to remove 
       the unnecessary cable(s). After ensuring that ONLY ONE 
       programming cable is attached, run \"./restore_my_flash\" again.  
       
       For details on jtagconfig, type jtagconfig help.
       
     ALSO, If you have JTAG USB devices other than your Altera
       programming cable (i.e. Cubic Cyclonium, etc.) you will 
       need to remove these devices and use jtagconfig to set 
       your cable configuration before performing the recovery.
       ";
   }
   # possible mistake, unattached or reversed cables
   elsif ($results =~ /$qpgm_scan_err/)
   {
     print "\n ERROR: PROGRAMMING CABLE IMPROPERLY CONNECTED TO TARGET:\n";
     print_likely_cause_of_programming_failure();
   }
   # Cable disconnected from PC or driver not installed properly
   elsif ( ($results =~ /$n2fp_no_cable_err/) || ($results =~ /$qpgm_no_cable_err/) )
   {
     print "\n ERROR: PROGRAMMING CABLE NOT FOUND:\n";
     print_likely_cause_of_programming_failure();
   }
   # JTAG ID error: the target FPGA doesn't match the .sof
   elsif ($results =~ /$qpgm_id_err/)
   {
     print "\n ERROR: PROGRAMMING FILE DOES NOT MATCH FPGA:\n";
     print_likely_cause_of_programming_failure();
   }
   else # V-E-R-N, yeah, my main man, Vern.
   {
     print "\n ERROR: REASON UNKNOWN.\n";
   }

   # if error flag did not get cleared, then there was a critical error
   if ($err_flag)
   {
     print "\n (RESTORE ABORTED) Please check above for error message(s) and try again or";
     print " \n    consult $user_doc";
     print " \n    for more information about restoring your Nios II Development Kit.\n\n";
     exit;                                     # complete ABORT, must ste-ril-ize er-ror!
   }

   return $err_flag;
}


sub display_header                 # first thing user sees
{

  print " 
     ***********************************************
          *-----------------------------------*
          |  ALTERA  Nios II Development Kit  |
          |   Flash Restore Program Rev.02    |
          *-----------------------------------*
     ***********************************************

 This program restores the default runtime code and
 \"Factory Configuration\" by \"flash_programming\" it  
 back into the $full_kit_name.

 If the flash files that were shipped with the kit 
 have been written over or otherwise missing, the flash 
 contents will be restored to the original state.
 
 ";
 
 return;
}


sub display_help # somebody needs some 'splainin'
{
print "
 
  * * * * * * * * * * * * * * * * *
    Flash Restore  - HELP Message:
  * * * * * * * * * * * * * * * * *

  You were supposed to press \"yes\" to restore your
  $full_kit_name.

  Perhaps, you do not want to restore your kit
  OR, you do not know if this is a good idea?

  So, Flash is untouched and the restore is aborted.

  Your $full_kit_name
  ships with a factory demonstration web_server
  design located in flash memory at a very specific
  flash offset address AND an associated HW image at a
  specific nearby flash address. These cause your kit to
  boot and run the demo automatically upon power on reset.

  If you have been writing over these values, either
  by mistake or deliberately, and the factory design
  no longer functions as it did \"out-of-the-box\",
  you may use this script to restore the original
  demo web server design and runtime code to the original
  flash locations by simply typing \"\.\\restore_my_flash\"

  If you are still uncertain, please consult
  $user_doc
  for details regarding factory recovery proceedures.

  If you are certain that you wish to restore your
  $full_kit_name to
  the factory \"out-of-the-box\" state,
  please run this script again by typing:\"\.\\restore_my_flash\"
  from the correct directory, then choose \"yes\" when prompted.
  Your flash files will be restored to factory condition.

  ";

 return;
}


sub display_restore              # Preparing the user to restore
{
    my $restore_steps = "";

    if ($short_kit_name =~ /^cyclone_1c12_eval$/)
    {
        $restore_steps = "
  1. IMPORTANT: Be sure that NO programming cables are attached
      to your PC.

  2. Connect a USB cable to your $full_kit_name.\n";
    }
    else
    {
      $restore_steps = "
   1. IMPORTANT: Be sure that ONLY ONE programming cable is attached
      to your PC and that it is connected to the J24 JTAG connector
      labeled \"Byteblaster\" of your
      $full_kit_name.

   2. Be sure DC power is applied to J26 labeled \"POWER\".\n";
    }

  print "

  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
              PREPARING TO RESTORE FLASH on your                    
         $full_kit_name   
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

  INSTRUCTIONS:
$restore_steps
      ALSO, If you have JTAG USB devices other than your
      programming cable (i.e. Cubic Cyclonium, etc.) you will
      need to remove these devices and use jtagconfig to set
      your cable configuration before performing the recovery.\n";
 return;
}


sub display_done  # Prepareing the user to restore
{
   print "\n  Restored flash successfully on your 
        $full_kit_name.
    
  NOTES: Restoring flash has not changed the network settings for your kit.
  However, if the factory program does not find valid network settings,
  the user will be automatically instructed as to how to reset them.
  Also, Restoring flash has not changed your \"User\" flash region.
  
  For more details on restoring the Nios II Development Kit, please consult 
  $user_doc    

  PRESS the $board_reconfig_sw to boot the factory design.
    
  ";
 
 return;
}

sub do_restore          # pay attention while restoring, detect and warn user about errors
{

  my  $keyIn = ""; # going to need one of these

  # first reason to quit early: no kit from arg list or default dir search
  if ($short_kit_name = find_kit_name) # find the kit name in the working dir string
  {
      $restore_flash   = "restore_" . $short_kit_name . ".flash"; # dumb it down to immediate dir,
      $target_sof = "restore_" . $short_kit_name . ".sof";        # specific sof and flash files

      if ($short_kit_name =~ "cyclone_1c12_eval")                 # the 1c12 eval has a second flash file for epcs
      {
          $restore_epcs_flash = "restore_" . $short_kit_name . ".epcsflash";
      }
  }
  else
  {
      display_kit_name_error; # explain the error to the user
      display_usage; # say something smart and polite
      return;
  }
  
  # 2nd reason to quit early: kit vars fail
  if ($full_kit_name = set_kit_vars($short_kit_name)) # use the short name to get the full name
  {
      $target_sof = $target_sof; # create the full target_sof path and file name
  }
  else
  {
     print " RESTORE ABORTED: Could not find correct development kit VARS.\n";
     print " Please be sure you are in the correct kit directory and try again.\n";
     display_usage;
     return;
  }
   
 
 # talk to the user
 display_header;                     # display header message

 # last chance to back out
 print "\n ARE YOU SURE you want to restore the flash contents
 of your $full_kit_name?
  
 Press \"y\" for \"yes\" or any other key to Abort:, then press ENTER...> ";

 $keyIn = <STDIN>;               # get a key

 # anything but a yes aborts retoring flash
 if ( !($keyIn =~ /y/) and !($keyIn =~ /Y/) )
 {
   print "\n  (User ABORT Message): Flash Contents Untouched.\n";
   display_help;                 # display help message
   return;                       # ABORT this program now
 }
 else
 {
    display_restore;

    prompt("\n   3. Press Enter Key when ready to begin restoring flash... ", "Enter");

    print "\n RESTORING FLASH on your Altera $full_kit_name\n";
    print "\n NOTE: Restoring flash takes a few minutes. Please wait...\n\n";

    # download the flash programming SOF file
    print " STATUS: Attempting to download flash-programming .sof to target...\n";
    if (do_command("$qpgm --mode=jtag -o p\\\;$target_sof"))
    {
      print "\nError downloading progamming SOF file. Flash contents untouched.\n";
      return;
    }

    # run flash_programmer command on full flash image file
    print " STATUS: Attempting to program CFI flash with factory contents...\n";
    if (do_command("$n2fp --base=$base $restore_flash"))
    {
      print "\nError downloading flash contents. Flash may be modified.\n";
      return;
    }

    # the 1C12 eval board has two files to program; the second is for its EPCS flash
    if ($short_kit_name =~ /^cyclone_1c12_eval$/)
    {
      print " STATUS: Attempting to program EPCS flash with factory contents...\n";
      if (do_command("$n2fp --epcs --base=$epcs_base $restore_epcs_flash"))
      {
        print "\nError downloading flash contents. Flash may be modified.\n";
        return;
      }
    }

    display_done; # parting thoughts and de-briefing
  }

  return;
}

sub main
{
    do_restore();
}

main(@ARGV);

#END_OF_FILE
